@@ -18,11 +18,17 @@
 # Boston, MA 02111-1307, USA.
 
 """Utilities for obtaining braille strings for objects.  In general,
-there probably should be a singleton instance of the BrailleUtil
+there probably should be a singleton instance of the BrailleGenerator
 class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
+__id__        = "$Id: braillegenerator.py,v 1.45 2006/06/10 00:54:47 wwalker Exp $"
+__version__   = "$Revision: 1.45 $"
+__date__      = "$Date: 2006/06/10 00:54:47 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import math
 
 import atspi
@@ -152,8 +158,8 @@ class BrailleGenerator:
 
         if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
             text = ""
-            
-            result = atspi.getAcceleratorAndShortcut(obj)
+
+            result = util.getAcceleratorAndShortcut(obj)
 
             accelerator = result[0]
             #shortcut = result[1]
@@ -170,7 +176,7 @@ class BrailleGenerator:
             return text
         else:
             return None
-        
+
     def _getTextForAvailability(self, obj):
         """Returns a string to be displayed that describes the availability
         of the given object.
@@ -187,7 +193,7 @@ class BrailleGenerator:
             return _("grayed")
         else:
             return None
-        
+
     def _getTextForValue(self, obj):
         """Returns the text to be displayed for the object's current value.
 
@@ -283,7 +289,7 @@ class BrailleGenerator:
         text = util.appendString(text, util.getDisplayedText(obj))
         text = util.appendString(text, self._getTextForValue(obj))
         text = util.appendString(text, self._getTextForRole(obj))
-        
+
         regions = []
         componentRegion = braille.Component(obj, text)
         regions.append(componentRegion)
@@ -800,7 +806,18 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForPushButton", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        regions = []
+
+        text = ""
+        text = util.appendString(text, util.getDisplayedLabel(obj))
+        text = util.appendString(text, util.getDisplayedText(obj))
+        text = util.appendString(text, self._getTextForRole(obj))
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
 
     def _getBrailleRegionsForRadioButton(self, obj):
         """Get the braille for a radio button.  If the button already had
@@ -918,7 +935,7 @@ class BrailleGenerator:
         #text = util.appendString(text, util.getDisplayedText(obj))
         text = util.appendString(text, self._getTextForValue(obj))
         text = util.appendString(text, self._getTextForRole(obj))
-        
+
         regions = []
         componentRegion = braille.Component(obj, text)
         regions.append(componentRegion)
@@ -1000,8 +1017,31 @@ class BrailleGenerator:
                     break
 
         if len(regions) == 0:
-            regions = self._getDefaultBrailleRegions(
-                util.getRealActiveDescendant(obj))
+            if settings.readTableCellRow:
+                rowRegions = []
+                savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
+                settings.brailleVerbosityLevel = \
+                                             settings.VERBOSITY_LEVEL_BRIEF
+
+                parent = obj.parent
+                row = parent.table.getRowAtIndex(obj.index)
+                column = parent.table.getColumnAtIndex(obj.index)
+                focusRowRegion = None
+                for i in range(0, parent.table.nColumns):
+                    accRow = parent.table.getAccessibleAt(row, i)
+                    cell = atspi.Accessible.makeAccessible(accRow)
+                    [cellRegions, focusRegion] = \
+                        self._getDefaultBrailleRegions(util.getRealActiveDescendant(cell))
+                    if len(rowRegions):
+                        rowRegions.append(braille.Region(" "))
+                    rowRegions.append(cellRegions[0])
+                    if i == column:
+                        focusRowRegion = cellRegions[0]
+                regions = [rowRegions, focusRowRegion]
+                settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+            else:
+                regions = self._getDefaultBrailleRegions(
+                    util.getRealActiveDescendant(obj))
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
@@ -1012,7 +1052,7 @@ class BrailleGenerator:
             else:
                 regions[0].append(braille.Region(" " + _("collapsed")))
 
-        level = atspi.getNodeLevel(obj)
+        level = util.getNodeLevel(obj)
         if level >= 0:
             regions[0].append(braille.Region(" " + _("TREE LEVEL %d") \
                                              % (level + 1)))
@@ -1077,7 +1117,7 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForTerminal", obj)
 
         title = None
-        frame = atspi.getFrame(obj)
+        frame = util.getFrame(obj)
         if frame:
             title = frame.name
         if not title:
@@ -1085,7 +1125,7 @@ class BrailleGenerator:
 
         text = title
         text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
-        
+
         regions = []
         regions.append(braille.Region(text))
 
@@ -1176,11 +1216,14 @@ class BrailleGenerator:
         - groupChildren: if True, children of an object should be displayed
                          together with their parent, where each child is
                          separated by _ and the selected child is the Region
-                         that should get focus.
+                         that should get focus.  The default here is True,
+                         but this also is used in conjunction with
+                         settings.enableBrailleGrouping.
 
         Returns a list where the first element is a list of Regions to
         display and the second element is the Region which should get
-        focus.  """
+        focus.
+        """
 
         # If we want to group the children, first see if obj is a child of
         # something we like to group.  If so, then reset the obj to the obj's
@@ -1188,7 +1231,7 @@ class BrailleGenerator:
         # to group.  If all fails, we don't try grouping.
         #
         reallyGroupChildren = False
-        if groupChildren:
+        if settings.enableBrailleGrouping and groupChildren:
             parent = obj.parent
             isChild = parent \
                       and ((parent.role == rolenames.ROLE_MENU) \
