@@ -17,8 +17,13 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Provides a SpeechServer factory for gnome-speech drivers.
-"""
+"""Provides a SpeechServer factory for gnome-speech drivers."""
+
+__id__        = "$Id: gnomespeechfactory.py,v 1.34 2006/06/10 00:54:47 wwalker Exp $"
+__version__   = "$Revision: 1.34 $"
+__date__      = "$Date: 2006/06/10 00:54:47 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import bonobo
 
@@ -235,6 +240,7 @@ class SpeechServer(speechserver.SpeechServer):
         self.__driverName = driver.driverName
         self.__iid = iid
         self.__sayAll = None
+        self.__isSpeaking = False
 
     def __getRate(self, speaker):
         """Gets the voice-independent ACSS rate value of a voice."""
@@ -271,6 +277,26 @@ class SpeechServer(speechserver.SpeechServer):
 
         speaker.setParameterValue("rate", rate)
 
+    def __getPitch(self, speaker):
+        """Gets the voice-specific pitch setting for the
+           voice-independent ACSS pitch value.
+
+        Returns the voice-specific pitch setting.
+        """
+
+        if not self.__pitchInfo.has_key(speaker):
+            return 5.0
+
+        [minPitch, averagePitch, maxPitch] = self.__pitchInfo[speaker]
+        pitch = speaker.getParameterValue("pitch")
+        if pitch < averagePitch:
+            return 5.0 * (pitch - minPitch) / (averagePitch - minPitch)
+        elif pitch > averagePitch:
+            return 5.0 \
+                   + (5.0 * (pitch - averagePitch) / (maxPitch - averagePitch))
+        else:
+            return 5.0
+
     def __setPitch(self, speaker, acssPitch):
         """Determines the voice-specific pitch setting for the
         voice-independent ACSS pitch value.
@@ -381,11 +407,13 @@ class SpeechServer(speechserver.SpeechServer):
             if self.__sayAll.idForCurrentContext == id:
                 context = self.__sayAll.currentContext
                 if type == GNOME.Speech.speech_callback_speech_started:
+                    self.__isSpeaking = True
                     context.currentOffset = context.startOffset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
                         speechserver.SayAllContext.PROGRESS)
                 elif type == GNOME.Speech.speech_callback_speech_progress:
+                    self.__isSpeaking = True
                     context.currentOffset = context.startOffset + offset
                     self.__sayAll.progressCallback(
                         self.__sayAll.currentContext,
@@ -402,11 +430,18 @@ class SpeechServer(speechserver.SpeechServer):
                             self.__sayAll.currentContext.utterance,
                             acss)
                     except StopIteration:
+                        self.__isSpeaking = False
                         context.currentOffset = context.endOffset
                         self.__sayAll.progressCallback(
                             self.__sayAll.currentContext,
                             speechserver.SayAllContext.COMPLETED)
                         self.__sayAll = None
+        elif type == GNOME.Speech.speech_callback_speech_started:
+            self.__isSpeaking = True
+        elif type == GNOME.Speech.speech_callback_speech_progress:
+            self.__isSpeaking = True
+        elif type == GNOME.Speech.speech_callback_speech_ended:
+            self.__isSpeaking = False
 
     def getInfo(self):
         """Returns [driverName, serverId]
@@ -488,7 +523,8 @@ class SpeechServer(speechserver.SpeechServer):
         """
         i = 0
         for text in list:
-            self.speak(text, acss, interrupt and (i == 0))
+            if len(text) != 0:
+                self.speak(text, acss, interrupt and (i == 0))
             i += 1
 
     def __speak(self, text=None, acss=None, interrupt=True):
@@ -539,7 +575,10 @@ class SpeechServer(speechserver.SpeechServer):
         # because DECtalk wants to interpret these as inline DECtalk
         # commands.]]]
         #
-        text = text.replace("[", "[ ")
+        # [[[UPDATE: commented this out because gnome-speech-0.4.0 now
+        # tells DECtalk to do a [:phon off], reducing this problem.]]]
+        #
+        #text = text.replace("[", "[ ")
 
         try:
             # [[[TODO: WDW - back this stop out for now.  The problem is
@@ -550,6 +589,7 @@ class SpeechServer(speechserver.SpeechServer):
             #if interrupt:
             #    speaker.stop()
             self.__lastText = [text, acss]
+            self.__isSpeaking = True
             return speaker.say(text)
         except:
             # On failure, remember what we said, reset our connection to the
@@ -579,6 +619,10 @@ class SpeechServer(speechserver.SpeechServer):
 
         self.__speak(text, acss, interrupt)
 
+    def isSpeaking(self):
+        """"Returns True if the system is currently speaking."""
+        return self.__isSpeaking
+
     def sayAll(self, utteranceIterator, progressCallback):
         """Iterates through the given utteranceIterator, speaking
         each utterance one at a time.
@@ -599,6 +643,62 @@ class SpeechServer(speechserver.SpeechServer):
         except StopIteration:
             pass
 
+    def increaseSpeechPitch(self, step=0.5):
+        """Increases the speech pitch for the default voice.
+
+        Arguments:
+        - step: the pitch step increment.
+        """
+
+        # [[[TODO: richb - this is a hack for now.  Need to take min/max
+        # values in account, plus also need to take into account that
+        # different engines provide different pitch ranges.]]]
+
+        voices = settings.voices
+        acss = voices[settings.DEFAULT_VOICE]
+        speaker = self.__getSpeaker(acss)
+
+        pitchDelta = settings.speechPitchDelta
+
+        try:
+            pitch = min(10, self.__getPitch(speaker) + pitchDelta)
+            acss[ACSS.AVERAGE_PITCH] = pitch
+            self.__setPitch(speaker, pitch)
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "speech.increaseSpeechPitch: pitch is now " \
+                          " %d" % pitch)
+            self.speak(_("higher."))
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
+
+    def decreaseSpeechPitch(self, step=0.5):
+        """Decreases the speech pitch for the default voice.
+
+        Arguments:
+        - step: the pitch step decrement.
+        """
+
+        # [[[TODO: WDW - this is a hack for now.  Need to take min/max
+        # values in account, plus also need to take into account that
+        # different engines provide different rate ranges.]]]
+
+        voices = settings.voices
+        acss = voices[settings.DEFAULT_VOICE]
+        speaker = self.__getSpeaker(acss)
+
+        pitchDelta = settings.speechPitchDelta
+
+        try:
+            pitch = max(1, self.__getPitch(speaker) - pitchDelta)
+            acss[ACSS.AVERAGE_PITCH] = pitch
+            self.__setPitch(speaker, pitch)
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "speech.decreaseSpeechPitch: pitch is now " \
+                          " %d" % pitch)
+            self.speak(_("lower."))
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
+
     def increaseSpeechRate(self, step=5):
         """Increases the speech rate.
 
@@ -666,6 +766,7 @@ class SpeechServer(speechserver.SpeechServer):
                 self.__speakers[name].stop()
             except:
                 pass
+        self.__isSpeaking = False
 
     def shutdown(self):
         """Shuts down the speech engine."""
