@@ -17,6 +17,14 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+"""Provides a custom script for gcalctool."""
+
+__id__        = "$Id: gcalctool.py,v 1.10 2006/06/10 00:54:48 wwalker Exp $"
+__version__   = "$Revision: 1.10 $"
+__date__      = "$Date: 2006/06/10 00:54:48 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import orca.atspi as atspi
 import orca.braille as braille
 import orca.default as default
@@ -71,9 +79,9 @@ class Script(default.Script):
             # the text_view display. This can be found by looking for an
             # accessible object with a role of ROLE_EDITBAR.
             #
-            d = atspi.findByRole(event.source, rolenames.ROLE_TEXT)
+            d = util.findByRole(event.source, rolenames.ROLE_TEXT)
             if len(d) == 0:
-                d = atspi.findByRole(event.source, rolenames.ROLE_EDITBAR)
+                d = util.findByRole(event.source, rolenames.ROLE_EDITBAR)
 
             # If d is an empty list at this point, we're unable to get the
             # gcalctool display. Inform the user.
